#include <TimerOne.h>

// Basic MIDI Controller code for reading all of the Arduino's digital and analogue inputs
// and sending them as MIDI messages to the host PC.
//
// Author: Michael Balzer
// 
// Revision History:
// Date        |  Change
// ---------------------------------------------------
// 2011-02-22  |  Initial Release
//
//
// This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License.
// See http://creativecommons.org/licenses/by-nc-sa/3.0/ for license details.

// Uncomment this line to send debug messages to the serial monitor
//#define DEBUG

// MIDI mapping taken from http://www.nortonmusic.com/midi_cc.html
#define MIDI_CC_MODULATION 0x01
#define MIDI_CC_BREATH 0x02
#define MIDI_CC_VOLUME 0x07
#define MIDI_CC_BALANCE 0x08
#define MIDI_CC_PAN 0x0A
#define MIDI_CC_EXPRESSION 0x0B
#define MIDI_CC_EFFECT1 0x0C
#define MIDI_CC_EFFECT2 0x0D

#define MIDI_CC_GENERAL1 0x0E
#define MIDI_CC_GENERAL2 0x0F
#define MIDI_CC_GENERAL3 0x10
#define MIDI_CC_GENERAL4 0x11
#define MIDI_CC_GENERAL5 0x12
#define MIDI_CC_GENERAL6 0x13
#define MIDI_CC_GENERAL7 0x14
#define MIDI_CC_GENERAL8 0x15
#define MIDI_CC_GENERAL9 0x16
#define MIDI_CC_GENERAL10 0x17
#define MIDI_CC_GENERAL11 0x18
#define MIDI_CC_GENERAL12 0x19
#define MIDI_CC_GENERAL13 0x1A
#define MIDI_CC_GENERAL14 0x1B
#define MIDI_CC_GENERAL15 0x1C
#define MIDI_CC_GENERAL16 0x1D
#define MIDI_CC_GENERAL17 0x1E
#define MIDI_CC_GENERAL18 0x1F

#define MIDI_CC_GENERAL1_FINE 0x2E
#define MIDI_CC_GENERAL2_FINE 0x2F
#define MIDI_CC_GENERAL3_FINE 0x30
#define MIDI_CC_GENERAL4_FINE 0x31
#define MIDI_CC_GENERAL5_FINE 0x32
#define MIDI_CC_GENERAL6_FINE 0x33
#define MIDI_CC_GENERAL7_FINE 0x34
#define MIDI_CC_GENERAL8_FINE 0x35
#define MIDI_CC_GENERAL9_FINE 0x36
#define MIDI_CC_GENERAL10_FINE 0x37
#define MIDI_CC_GENERAL11_FINE 0x38
#define MIDI_CC_GENERAL12_FINE 0x39
#define MIDI_CC_GENERAL13_FINE 0x3A
#define MIDI_CC_GENERAL14_FINE 0x3B
#define MIDI_CC_GENERAL15_FINE 0x3C
#define MIDI_CC_GENERAL16_FINE 0x3D
#define MIDI_CC_GENERAL17_FINE 0x3E
#define MIDI_CC_GENERAL18_FINE 0x3F

#define MIDI_CC_SUSTAIN 0x40
#define MIDI_CC_REVERB 0x5B
#define MIDI_CC_CHORUS 0x5D
#define MIDI_CC_CONTROL_OFF 0x79
#define MIDI_CC_NOTES_OFF 0x78



// Comment this line out to disable button debounce logic.
// See http://arduino.cc/en/Tutorial/Debounce what debouncing is used for.
#define DEBOUNCE
// Debounce time length in milliseconds
#define DEBOUNCE_LENGTH 5

// Comment this line out to disable analogue filtering
#define ANALOGUE_FILTER
// A knob or slider movement must initially exceed this value to be recognised as an input. Note that it is
// for a 7-bit MIDI value.
#define FILTER_AMOUNT 2
// Timeout is in microseconds
#define ANALOGUE_INPUT_CHANGE_TIMEOUT 1000000

// Number of digital inputs. Can be anywhere from 0 to 18.
#define NUM_DI 12
// Number of analogue inputs. Can be anywhere from 0 to 6.
#define NUM_AI 6

// Array containing a mapping of digital pins to channel index. This array size must match NUM_DI above.
byte digitalInputMapping[NUM_DI] = { 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13 };
// Array containing a mapping of analogue pins to channel index. This array size must match NUM_AI above.
byte analogueInputMapping[NUM_AI] = { A0, A1, A2, A3, A4, A5 };

// Contains the current state of the digital inputs.
byte digitalInputs[NUM_DI];
// Contains the current value of the analogue inputs.
byte analogueInputs[NUM_AI];

// Variable to hold temporary digital reads, used for debounce logic.
byte tempDigitalInput;
// Variable to hold temporary analogue values, used for analogue filtering logic.
byte tempAnalogueInput;

// Preallocate the for loop index so we don't keep reallocating it for every program iteration.
int i = 0;
// Variable to hold difference between current and new analogue input values.
int analogueDiff = 0;
// This is used as a flag to indicate that an analogue input is changing.
boolean analogueInputChanging;

void setup()
{
  // Enable serial I/O at 57600 kbps. This is faster than the standard MIDI rate of 31250 kbps.
  // The PC application which we connect to will automatically take the higher sample rate and send MIDI
  // messages out at the correct rate. We only send things faster in case there is any latency.
  Serial.begin(115200);
  
  // Initialise each digital input channel.
  for (i = 0; i < NUM_DI; i++)
  {
    // Set the pin direction to input.
    pinMode(digitalInputMapping[i], INPUT);
    // Don't enable pullup resistor on pin 13, as the LED and resistor will always pull it low, meaning the input won't work.
    // Instead an external pulldown resistor must be used on pin 13.
    // NOTE: This will cause all of the high/low logic for pin 13 to be inverted.
    if (digitalInputMapping[i] != 13)
    {
      // Enable the pull-up resistor. This call must come after the above pinMode call.
      digitalWrite(digitalInputMapping[i], HIGH);
    }
    
    // Initialise the digital state with a read to the input pin.
    digitalInputs[i] = digitalRead(digitalInputMapping[i]);
  }
  
  // Initialise each analogue input channel.
  for (i = 0; i < NUM_AI; i++)
  {
    // Set the pin direction to input.
    pinMode(analogueInputMapping[i], INPUT);
    
    // Initialise the analogue value with a read to the input pin.
    analogueInputs[i] = analogRead(analogueInputMapping[i]);
  }
  
  // Assume no analogue inputs are active
  analogueInputChanging = false;
  
  // This timer runs every 1 second
  Timer1.initialize(ANALOGUE_INPUT_CHANGE_TIMEOUT);
  // When the timer expires, call this function
  Timer1.attachInterrupt(analogueInputStopped);
  // Start the timer
  Timer1.start();
}



void loop()
{
  for (i = 0; i < NUM_DI; i++)
  {
    // Read the current state of the digital input and store it temporarily.
    tempDigitalInput = digitalRead(digitalInputMapping[i]);
    
    // Check if the last state is different to the current state.
    if (digitalInputs[i] != tempDigitalInput)
    {
      #ifdef DEBOUNCE
      // Wait for a short period of time, and then take a second reading from the input pin.
      delay(DEBOUNCE_LENGTH);
      // If the second reading is the same as the initial reading, assume it must be true.
      if (tempDigitalInput == digitalRead(digitalInputMapping[i]))
      {
      #endif
        // Record the new digital input state.
        digitalInputs[i] = tempDigitalInput;
        
        // Moved from HIGH to LOW (button pressed)
        if (digitalInputs[i] == 0)
        {
          // All the digital inputs use pullup resistors, except pin 13 so the logic is inverted
          if (digitalInputMapping[i] != 13)
          {
            noteOn(0, 0x00 + i, 0x7F); // Channel 1, middle C, maximum velocity
          }
          else
          {
            noteOff(0, 0x00 + i); // Channel 1, middle C
          }
        }
        // Moved from LOW to HIGH (button released)
        else
        {
          // All the digital inputs use pullup resistors, except pin 13 so the logic is inverted
          if (digitalInputMapping[i] != 13)
          {
            noteOff(0, 0x00 + i); // Channel 1, middle C
          }
          else
          {
            noteOn(0, 0x00 + i, 0x7F); // Channel 1, middle C, maximum velocity
          }
        }
      #ifdef DEBOUNCE
      }
      #endif
    }
  }
  
  /*
   * Analogue input logic:
   * The Arduino uses a 10-bit (0-1023) analogue to digital converter (ADC) on each of its analogue inputs.
   * The ADC isn't very high resolution, so if a pot is in a position such that the output voltage is 'between'
   * what it can detect (say 2.505V or about 512.5 on a scale of 0-1023) then the value read will constantly
   * fluctuate between two integers (in this case 512 and 513).
   *
   * If we're simply looking for a change in the analogue input value like in the digital case above, then
   * there will be cases where the value is always changing, even though the physical input isn't being moved.
   * This will in turn send out a constant stream of MIDI messages to the connected software which may be problematic.
   *
   * To combat this, we require that the analogue input value must change by a certain threshold amount before
   * we register that it is actually changing. This is good in avoiding a constantly fluctuating value, but has
   * the negative effect of a reduced input resolution. For example if the threshold amount was 2 and we slowly moved
   * a slider through it's full range, we would only detect every second value as a change, in effect reducing the
   * already small 7-bit MIDI value to a 6-bit MIDI value.
   *
   * To get around this problem but still use the threshold logic, a timer is used. Initially the analogue input
   * must exceed the threshold to be detected as an input. Once this occurs, we then read every value coming from the
   * analogue input (not just those exceeding a threshold) giving us full 7-bit resolution. At the same time the
   * timer is started. This timer is used to keep track of whether an input hasn't been moved for a certain time
   * period. If it has been moved, the timer is restarted. If no movement occurs the timer is just left to run. When
   * the timer expires the analogue input is assumed to be no longer moving. Subsequent movements must exceed the
   * threshold amount.
   */
  for (i = 0; i < NUM_AI; i++)
  {
    // Read the analogue input pin, dividing it by 8 so the 10-bit ADC value (0-1023) is converted to a 7-bit MIDI value (0-127).
    tempAnalogueInput = analogRead(analogueInputMapping[i]) / 8;
    
    #ifdef ANALOGUE_FILTER
    // Take the absolute value of the difference between the curent and new values 
    analogueDiff = abs(tempAnalogueInput - analogueInputs[i]);
    // Only continue if the threshold was exceeded, or the input was already changing
    if ((analogueDiff > 0 && analogueInputChanging == true) || analogueDiff >= FILTER_AMOUNT)
    {
    #else
    if (analogueInputs[i] != tempAnalogueInput)
    {
    #endif
      // If the the analogue input wasn't changing, we need to start the timer again
      if (analogueInputChanging == false)
      {
        Timer1.start();
      }
      // The analogue input was moving, so restart the timer. Only restart it if we're sure the input isn't 'between' a value
      // ie. It's moved more than FILTER_AMOUNT
      else if (analogueDiff >= FILTER_AMOUNT)
      {
        Timer1.restart();
      }
      
      // The analogue input is moving
      analogueInputChanging = true;
      
      // Record the new analogue value
      analogueInputs[i] = tempAnalogueInput;
      
      // Send the analogue value out on the general MIDI CC (see definitions at beginning of this file)
      controlChange(0, MIDI_CC_GENERAL1 + i, analogueInputs[i]);
    }
  }
}

// Send a MIDI note on message
void noteOn(int channel, int pitch, int velocity)
{
  // 0x90 is the first of 16 note on channels
  channel += 0x90;
  
  // Ensure we're between channels 1 and 16 for a note on message
  if (channel >= 0x90 && channel <= 0x9F)
  {
    #ifdef DEBUG
      Serial.print("Button pressed: ");
      Serial.println(pitch);
    #else
      Serial.print(channel, BYTE);
      Serial.print(pitch, BYTE);
      Serial.print(velocity, BYTE);
    #endif
  }
}

// Send a MIDI note off message
void noteOff(int channel, int pitch)
{
  // 0x80 is the first of 16 note off channels
  channel += 0x80;
  
  // Ensure we're between channels 1 and 16 for a note off message
  if (channel >= 0x80 && channel <= 0x8F)
  {
    #ifdef DEBUG
      Serial.print("Button released: ");
      Serial.println(pitch);
    #else 
      Serial.print(channel, BYTE);
      Serial.print(pitch, BYTE);
      Serial.print(0x00, BYTE);
    #endif
  }
}

// Send a MIDI control change message
void controlChange(int channel, int control, int value)
{
  // 0xB0 is the first of 16 control change channels
  channel += 0xB0;
  
  // Ensure we're between channels 1 and 16 for a CC message
  if (channel >= 0xB0 && channel <= 0xBF)
  {
    #ifdef DEBUG
      Serial.print(control - MIDI_CC_GENERAL1);
      Serial.print(": ");
      Serial.println(value);
    #else
      Serial.print(channel, BYTE);
      Serial.print(control, BYTE);
      Serial.print(value, BYTE);
    #endif
  }
}

// The timer has expired
void analogueInputStopped()
{
  // Stop the timer so it doesn't repeatedly call this function.
  Timer1.stop();
  // The analogue input is no longer moving
  analogueInputChanging = false;
}
